@@ -13,16 +13,17 @@
 # limitations under the License.
 
 """High-level options interface. This allows defining options just once that
-can be specified from the command line and the noxfile, easily used in tests,
+can be specified from the command line and the Noxfile, easily used in tests,
 and surfaced in documentation."""
 
+from __future__ import annotations
 
 import argparse
 import collections
 import functools
 from argparse import ArgumentError as ArgumentError
 from argparse import ArgumentParser, Namespace
-from typing import Any, Callable, List, Optional, Tuple, Union
+from typing import Any, Callable
 
 import argcomplete
 
@@ -36,6 +37,8 @@ class OptionGroup:
         kwargs: Passed through to``ArgumentParser.add_argument_group``.
     """
 
+    __slots__ = ("name", "args", "kwargs")
+
     def __init__(self, name: str, *args: Any, **kwargs: Any) -> None:
         self.name = name
         self.args = args
@@ -77,14 +80,18 @@ class Option:
         self,
         name: str,
         *flags: str,
-        group: Optional[OptionGroup],
-        help: Optional[str] = None,
+        group: OptionGroup | None,
+        help: str | None = None,
         noxfile: bool = False,
-        merge_func: Optional[Callable[[Namespace, Namespace], Any]] = None,
-        finalizer_func: Optional[Callable[[Any, Namespace], Any]] = None,
-        default: Union[Any, Callable[[], Any]] = None,
+        merge_func: Callable[[Namespace, Namespace], Any] | None = None,
+        finalizer_func: Callable[[Any, Namespace], Any] | None = None,
+        default: bool
+        | str
+        | None
+        | list[str]
+        | Callable[[], bool | str | None | list[str]] = None,
         hidden: bool = False,
-        completer: Optional[Callable[..., List[str]]] = None,
+        completer: Callable[..., list[str]] | None = None,
         **kwargs: Any,
     ) -> None:
         self.name = name
@@ -100,7 +107,7 @@ class Option:
         self._default = default
 
     @property
-    def default(self) -> Optional[Union[bool, str]]:
+    def default(self) -> bool | str | None | list[str]:
         if callable(self._default):
             return self._default()
         return self._default
@@ -112,7 +119,7 @@ def flag_pair_merge_func(
     command_args: Namespace,
     noxfile_args: Namespace,
 ) -> bool:
-    """Merge function for flag pairs. If the flag is set in the noxfile or
+    """Merge function for flag pairs. If the flag is set in the Noxfile or
     the command line params, return ``True`` *unless* the disable flag has been
     specified on the command-line.
 
@@ -154,13 +161,14 @@ def flag_pair_merge_func(
 
 def make_flag_pair(
     name: str,
-    enable_flags: Union[Tuple[str, str], Tuple[str]],
-    disable_flags: Tuple[str],
+    enable_flags: tuple[str, str] | tuple[str],
+    disable_flags: tuple[str, str] | tuple[str],
+    default: bool = False,
     **kwargs: Any,
-) -> Tuple[Option, Option]:
+) -> tuple[Option, Option]:
     """Returns two options - one to enable a behavior and another to disable it.
 
-    The positive option is considered to be available to the noxfile, as
+    The positive option is considered to be available to the Noxfile, as
     there isn't much point in doing flag pairs without it.
     """
     disable_name = f"no_{name}"
@@ -171,6 +179,7 @@ def make_flag_pair(
         *enable_flags,
         noxfile=True,
         merge_func=functools.partial(flag_pair_merge_func, name, disable_name),
+        default=default,
         **kwargs,
     )
 
@@ -191,10 +200,10 @@ class OptionSet:
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         self.parser_args = args
         self.parser_kwargs = kwargs
-        self.options: "collections.OrderedDict[str, Option]" = collections.OrderedDict()
-        self.groups: "collections.OrderedDict[str, OptionGroup]" = (
-            collections.OrderedDict()
-        )
+        self.options: collections.OrderedDict[str, Option] = collections.OrderedDict()
+        self.groups: collections.OrderedDict[
+            str, OptionGroup
+        ] = collections.OrderedDict()
 
     def add_options(self, *args: Option) -> None:
         """Adds a sequence of Options to the OptionSet.
@@ -228,7 +237,7 @@ class OptionSet:
         }
 
         for option in self.options.values():
-            if option.hidden is True:
+            if option.hidden:
                 continue
 
             # Every option must have a group (except for hidden options)
@@ -306,7 +315,7 @@ class OptionSet:
     def merge_namespaces(
         self, command_args: Namespace, noxfile_args: Namespace
     ) -> None:
-        """Merges the command-line options with the noxfile options."""
+        """Merges the command-line options with the Noxfile options."""
         command_args_copy = Namespace(**vars(command_args))
         for name, option in self.options.items():
             if option.merge_func:
