@@ -12,16 +12,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 import argparse
 import functools
 import os
 import sys
-from typing import Any, List, Optional, Sequence, Union
+from typing import Any, Sequence
 
 from nox import _option_set
 from nox.tasks import discover_manifest, filter_manifest, load_nox_module
 
-"""All of nox's configuration options."""
+"""All of Nox's configuration options."""
 
 options = _option_set.OptionSet(
     description="Nox is a Python automation toolkit.", add_help=False
@@ -46,7 +48,8 @@ options.add_groups(
     _option_set.OptionGroup(
         "environment",
         "Environment options",
-        "These arguments are used to control Nox's creation and usage of virtual environments.",
+        "These arguments are used to control Nox's creation and usage of virtual"
+        " environments.",
     ),
     _option_set.OptionGroup(
         "execution",
@@ -63,7 +66,7 @@ options.add_groups(
 
 def _sessions_and_keywords_merge_func(
     key: str, command_args: argparse.Namespace, noxfile_args: argparse.Namespace
-) -> List[str]:
+) -> list[str]:
     """Only return the Noxfile value for sessions/keywords if neither sessions
     or keywords are specified on the command-line.
 
@@ -76,14 +79,14 @@ def _sessions_and_keywords_merge_func(
         noxfile_Args (_option_set.Namespace): The options specified in the
             Noxfile."""
     if not command_args.sessions and not command_args.keywords:
-        return getattr(noxfile_args, key)
-    return getattr(command_args, key)
+        return getattr(noxfile_args, key)  # type: ignore[no-any-return]
+    return getattr(command_args, key)  # type: ignore[no-any-return]
 
 
 def _default_venv_backend_merge_func(
     command_args: argparse.Namespace, noxfile_args: argparse.Namespace
 ) -> str:
-    """Merge default_venv_backend from command args and nox file. Default is "virtualenv".
+    """Merge default_venv_backend from command args and Noxfile. Default is "virtualenv".
 
     Args:
         command_args (_option_set.Namespace): The options specified on the
@@ -101,7 +104,7 @@ def _default_venv_backend_merge_func(
 def _force_venv_backend_merge_func(
     command_args: argparse.Namespace, noxfile_args: argparse.Namespace
 ) -> str:
-    """Merge force_venv_backend from command args and nox file. Default is None.
+    """Merge force_venv_backend from command args and Noxfile. Default is None.
 
     Args:
         command_args (_option_set.Namespace): The options specified on the
@@ -120,7 +123,7 @@ def _force_venv_backend_merge_func(
         else:
             return "none"
     else:
-        return command_args.force_venv_backend or noxfile_args.force_venv_backend
+        return command_args.force_venv_backend or noxfile_args.force_venv_backend  # type: ignore[no-any-return]
 
 
 def _envdir_merge_func(
@@ -137,7 +140,7 @@ def _envdir_merge_func(
     return command_args.envdir or noxfile_args.envdir or ".nox"
 
 
-def _sessions_default() -> Optional[List[str]]:
+def _sessions_default() -> list[str] | None:
     """Looks at the NOXSESSION env var to set the default value for sessions."""
     nox_env = os.environ.get("NOXSESSION")
     env_sessions = nox_env.split(",") if nox_env else None
@@ -154,15 +157,15 @@ def _color_finalizer(value: bool, args: argparse.Namespace) -> bool:
     Returns:
         The new value for the "color" option.
     """
-    if args.forcecolor is True and args.nocolor is True:
+    if args.forcecolor and args.nocolor:
         raise _option_set.ArgumentError(
             None, "Can not specify both --no-color and --force-color."
         )
 
-    if args.forcecolor is True:
+    if args.forcecolor:
         return True
 
-    if args.nocolor is True:
+    if args.nocolor:
         return False
 
     return sys.stdout.isatty()
@@ -186,7 +189,7 @@ def _R_finalizer(value: bool, args: argparse.Namespace) -> bool:
 
 def _posargs_finalizer(
     value: Sequence[Any], args: argparse.Namespace
-) -> Union[Sequence[Any], List[Any]]:
+) -> Sequence[Any] | list[Any]:
     """Removes the leading "--"s in the posargs array (if any) and asserts that
     remaining arguments came after a "--".
     """
@@ -212,7 +215,7 @@ def _posargs_finalizer(
 
 def _session_completer(
     prefix: str, parsed_args: argparse.Namespace, **kwargs: Any
-) -> List[str]:
+) -> list[str]:
     global_config = parsed_args
     module = load_nox_module(global_config)
     manifest = discover_manifest(module, global_config)
@@ -282,6 +285,15 @@ options.add_options(
         merge_func=functools.partial(_sessions_and_keywords_merge_func, "keywords"),
         help="Only run sessions that match the given expression.",
     ),
+    _option_set.Option(
+        "tags",
+        "-t",
+        "--tags",
+        group=options.groups["sessions"],
+        noxfile=True,
+        nargs="*",
+        help="Only run sessions with the given tags.",
+    ),
     _option_set.Option(
         "posargs",
         "posargs",
@@ -315,8 +327,11 @@ options.add_options(
         group=options.groups["environment"],
         noxfile=True,
         merge_func=_default_venv_backend_merge_func,
-        help="Virtual environment backend to use by default for nox sessions, this is ``'virtualenv'`` by default but "
-        "any of ``('virtualenv', 'conda', 'mamba', 'venv')`` are accepted.",
+        help=(
+            "Virtual environment backend to use by default for Nox sessions, this is"
+            " ``'virtualenv'`` by default but any of ``('virtualenv', 'conda', 'mamba',"
+            " 'venv')`` are accepted."
+        ),
         choices=["none", "virtualenv", "conda", "mamba", "venv"],
     ),
     _option_set.Option(
@@ -326,9 +341,12 @@ options.add_options(
         group=options.groups["environment"],
         noxfile=True,
         merge_func=_force_venv_backend_merge_func,
-        help="Virtual environment backend to force-use for all nox sessions in this run, overriding any other venv "
-        "backend declared in the nox file and ignoring the default backend. Any of ``('virtualenv', 'conda', 'mamba', 'venv')`` "
-        "are accepted.",
+        help=(
+            "Virtual environment backend to force-use for all Nox sessions in this run,"
+            " overriding any other venv backend declared in the Noxfile and ignoring"
+            " the default backend. Any of ``('virtualenv', 'conda', 'mamba', 'venv')``"
+            " are accepted."
+        ),
         choices=["none", "virtualenv", "conda", "mamba", "venv"],
     ),
     _option_set.Option(
@@ -337,13 +355,18 @@ options.add_options(
         group=options.groups["environment"],
         default=False,
         action="store_true",
-        help="Runs the selected sessions directly on the current interpreter, without creating a venv. This is an alias "
-        "for '--force-venv-backend none'.",
+        help=(
+            "Runs the selected sessions directly on the current interpreter, without"
+            " creating a venv. This is an alias for '--force-venv-backend none'."
+        ),
     ),
     *_option_set.make_flag_pair(
         "reuse_existing_virtualenvs",
         ("-r", "--reuse-existing-virtualenvs"),
-        ("--no-reuse-existing-virtualenvs",),
+        (
+            "-N",
+            "--no-reuse-existing-virtualenvs",
+        ),
         group=options.groups["environment"],
         help="Re-use existing virtualenvs instead of recreating them.",
     ),
@@ -365,7 +388,7 @@ options.add_options(
         "--noxfile",
         group=options.groups["general"],
         default="noxfile.py",
-        help="Location of the Python file containing nox sessions.",
+        help="Location of the Python file containing Nox sessions.",
     ),
     _option_set.Option(
         "envdir",
@@ -373,7 +396,7 @@ options.add_options(
         noxfile=True,
         merge_func=_envdir_merge_func,
         group=options.groups["environment"],
-        help="Directory where nox will store virtualenvs, this is ``.nox`` by default.",
+        help="Directory where Nox will store virtualenvs, this is ``.nox`` by default.",
     ),
     _option_set.Option(
         "extra_pythons",
@@ -390,8 +413,8 @@ options.add_options(
         group=options.groups["python"],
         nargs="*",
         help=(
-            "Run sessions with the given interpreters instead of those listed in the Noxfile."
-            " This is a shorthand for ``--python=X.Y --extra-python=X.Y``."
+            "Run sessions with the given interpreters instead of those listed in the"
+            " Noxfile. This is a shorthand for ``--python=X.Y --extra-python=X.Y``."
         ),
         finalizer_func=_force_pythons_finalizer,
     ),
@@ -408,13 +431,17 @@ options.add_options(
         ("--no-error-on-missing-interpreters",),
         group=options.groups["execution"],
         help="Error instead of skipping sessions if an interpreter can not be located.",
+        default="CI" in os.environ,
     ),
     *_option_set.make_flag_pair(
         "error_on_external_run",
         ("--error-on-external-run",),
         ("--no-error-on-external-run",),
         group=options.groups["execution"],
-        help="Error if run() is used to execute a program that isn't installed in a session's virtualenv.",
+        help=(
+            "Error if run() is used to execute a program that isn't installed in a"
+            " session's virtualenv."
+        ),
     ),
     _option_set.Option(
         "install_only",
@@ -447,7 +474,10 @@ options.add_options(
         "--non-interactive",
         group=options.groups["execution"],
         action="store_true",
-        help="Force session.interactive to always be False, even in interactive sessions.",
+        help=(
+            "Force session.interactive to always be False, even in interactive"
+            " sessions."
+        ),
     ),
     _option_set.Option(
         "nocolor",
@@ -480,7 +510,7 @@ options.add_options(
         "invoked_from",
         group=None,
         hidden=True,
-        default=lambda: os.getcwd(),
+        default=os.getcwd,
     ),
 )
 
