@@ -12,12 +12,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 import os
 import shlex
+import shutil
 import sys
-from typing import Any, Iterable, List, Optional, Sequence, Union
-
-import py
+from typing import Any, Iterable, Sequence
 
 from nox.logger import logger
 from nox.popen import popen
@@ -31,31 +32,27 @@ else:  # pragma: no cover
 class CommandFailed(Exception):
     """Raised when an executed command returns a non-success status code."""
 
-    def __init__(self, reason: Optional[str] = None) -> None:
+    def __init__(self, reason: str | None = None) -> None:
         super().__init__(reason)
         self.reason = reason
 
 
-def which(program: str, paths: Optional[List[str]]) -> str:
+def which(program: str, paths: list[str] | None) -> str:
     """Finds the full path to an executable."""
-    full_path = None
-
-    if paths:
-        full_path = py.path.local.sysfind(program, paths=paths)
+    if paths is not None:
+        full_path = shutil.which(program, path=os.pathsep.join(paths))
+        if full_path:
+            return full_path
 
+    full_path = shutil.which(program)
     if full_path:
-        return full_path.strpath
-
-    full_path = py.path.local.sysfind(program)
-
-    if full_path:
-        return full_path.strpath
+        return full_path
 
     logger.error(f"Program {program} not found.")
     raise CommandFailed(f"Program {program} not found")
 
 
-def _clean_env(env: Optional[dict]) -> Optional[dict]:
+def _clean_env(env: dict[str, str] | None) -> dict[str, str] | None:
     if env is None:
         return None
 
@@ -69,21 +66,20 @@ def _clean_env(env: Optional[dict]) -> Optional[dict]:
 
 
 def _shlex_join(args: Sequence[str]) -> str:
-    # shlex.join() was added in Python 3.8
-    return " ".join(shlex.quote(arg) for arg in args)
+    return " ".join(shlex.quote(os.fspath(arg)) for arg in args)
 
 
 def run(
     args: Sequence[str],
     *,
-    env: Optional[dict] = None,
+    env: dict[str, str] | None = None,
     silent: bool = False,
-    paths: Optional[List[str]] = None,
-    success_codes: Optional[Iterable[int]] = None,
+    paths: list[str] | None = None,
+    success_codes: Iterable[int] | None = None,
     log: bool = True,
-    external: Union[Literal["error"], bool] = False,
+    external: Literal["error"] | bool = False,
     **popen_kws: Any,
-) -> Union[str, bool]:
+) -> str | bool:
     """Run a command-line program."""
 
     if success_codes is None:
@@ -103,14 +99,16 @@ def run(
         if is_external_tool:
             if external == "error":
                 logger.error(
-                    f"Error: {cmd} is not installed into the virtualenv, it is located at {cmd_path}. "
-                    "Pass external=True into run() to explicitly allow this."
+                    f"Error: {cmd} is not installed into the virtualenv, it is located"
+                    f" at {cmd_path}. Pass external=True into run() to explicitly allow"
+                    " this."
                 )
                 raise CommandFailed("External program disallowed.")
             elif external is False:
                 logger.warning(
-                    f"Warning: {cmd} is not installed into the virtualenv, it is located at {cmd_path}. This might cause issues! "
-                    "Pass external=True into run() to silence this message."
+                    f"Warning: {cmd} is not installed into the virtualenv, it is"
+                    f" located at {cmd_path}. This might cause issues! Pass"
+                    " external=True into run() to silence this message."
                 )
 
     env = _clean_env(env)
