@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 import logging
 from typing import Any, cast
 
@@ -45,7 +47,7 @@ class NoxFormatter(logging.Formatter):
         return super().format(record)
 
 
-class NoxColoredFormatter(ColoredFormatter):
+class NoxColoredFormatter(ColoredFormatter):  # type: ignore[misc]
     def __init__(
         self,
         datefmt: Any = None,
@@ -68,7 +70,7 @@ class NoxColoredFormatter(ColoredFormatter):
     def format(self, record: Any) -> str:
         if record.levelname == "OUTPUT":
             return self._simple_fmt.format(record)
-        return super().format(record)
+        return super().format(record)  # type: ignore[no-any-return]
 
 
 class LoggerWithSuccessAndOutput(logging.getLoggerClass()):  # type: ignore[misc]
@@ -95,7 +97,7 @@ logger = cast(LoggerWithSuccessAndOutput, logging.getLogger("nox"))
 
 
 def _get_formatter(color: bool, add_timestamp: bool) -> logging.Formatter:
-    if color is True:
+    if color:
         return NoxColoredFormatter(
             reset=True,
             log_colors={
@@ -110,8 +112,7 @@ def _get_formatter(color: bool, add_timestamp: bool) -> logging.Formatter:
             secondary_log_colors=None,
             add_timestamp=add_timestamp,
         )
-    else:
-        return NoxFormatter(add_timestamp=add_timestamp)
+    return NoxFormatter(add_timestamp=add_timestamp)
 
 
 def setup_logging(
