@@ -12,7 +12,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 import argparse
+import contextlib
 import enum
 import hashlib
 import os
@@ -20,35 +23,42 @@ import pathlib
 import re
 import sys
 import unicodedata
-import warnings
 from types import TracebackType
 from typing import (
+    TYPE_CHECKING,
     Any,
     Callable,
-    Dict,
+    Generator,
     Iterable,
-    List,
     Mapping,
-    Optional,
+    NoReturn,
     Sequence,
-    Tuple,
-    Type,
-    Union,
 )
 
-import py
-
 import nox.command
-from nox import _typing
 from nox._decorators import Func
 from nox.logger import logger
 from nox.virtualenv import CondaEnv, PassthroughEnv, ProcessEnv, VirtualEnv
 
-if _typing.TYPE_CHECKING:
+if TYPE_CHECKING:
     from nox.manifest import Manifest
 
 
-def _normalize_path(envdir: str, path: Union[str, bytes]) -> str:
+@contextlib.contextmanager
+def _chdir(path: str) -> Generator[None, None, None]:
+    """
+    Change the current working directory to the given path.
+    Follows python 3.11's chdir behaviour.
+    """
+    cwd = os.getcwd()
+    try:
+        os.chdir(path)
+        yield
+    finally:
+        os.chdir(cwd)
+
+
+def _normalize_path(envdir: str, path: str | bytes) -> str:
     """Normalizes a string to be a "safe" filesystem path for a virtualenv."""
     if isinstance(path, bytes):
         path = path.decode("utf-8")
@@ -69,13 +79,13 @@ def _normalize_path(envdir: str, path: Union[str, bytes]) -> str:
             logger.error(
                 f"The virtualenv path {full_path} is too long and will cause issues on "
                 "some environments. Use the --envdir path to modify where "
-                "nox stores virtualenvs."
+                "Nox stores virtualenvs."
             )
 
     return full_path
 
 
-def _dblquote_pkg_install_args(args: Tuple[str, ...]) -> Tuple[str, ...]:
+def _dblquote_pkg_install_args(args: tuple[str, ...]) -> tuple[str, ...]:
     """Double-quote package install arguments in case they contain '>' or '<' symbols"""
 
     # routine used to handle a single arg
@@ -87,7 +97,7 @@ def _dblquote_pkg_install_args(args: Tuple[str, ...]) -> Tuple[str, ...]:
             )
 
         if "<" in pkg_req_str or ">" in pkg_req_str:
-            if pkg_req_str[0] == '"' and pkg_req_str[-1] == '"':
+            if pkg_req_str[0] == pkg_req_str[-1] == '"':
                 # already double-quoted string
                 return pkg_req_str
             else:
@@ -119,18 +129,18 @@ class Status(enum.Enum):
 
 
 class _WorkingDirContext:
-    def __init__(self, dir: Union[str, os.PathLike]) -> None:
+    def __init__(self, dir: str | os.PathLike[str]) -> None:
         self._prev_working_dir = os.getcwd()
         os.chdir(dir)
 
-    def __enter__(self) -> "_WorkingDirContext":
+    def __enter__(self) -> _WorkingDirContext:
         return self
 
     def __exit__(
         self,
-        exc_type: Optional[Type[BaseException]],
-        exc_value: Optional[BaseException],
-        traceback: Optional[TracebackType],
+        exc_type: type[BaseException] | None,
+        exc_value: BaseException | None,
+        traceback: TracebackType | None,
     ) -> None:
         os.chdir(self._prev_working_dir)
 
@@ -144,11 +154,11 @@ class Session:
 
     __slots__ = ("_runner",)
 
-    def __init__(self, runner: "SessionRunner") -> None:
+    def __init__(self, runner: SessionRunner) -> None:
         self._runner = runner
 
     @property
-    def __dict__(self) -> "Dict[str, SessionRunner]":  # type: ignore[override]
+    def __dict__(self) -> dict[str, SessionRunner]:  # type: ignore[override]
         """Attribute dictionary for object inspection.
 
         This is needed because ``__slots__`` turns off ``__dict__`` by
@@ -163,13 +173,14 @@ class Session:
         return self._runner.friendly_name
 
     @property
-    def env(self) -> dict:
+    def env(self) -> dict[str, str]:
         """A dictionary of environment variables to pass into all commands."""
         return self.virtualenv.env
 
     @property
-    def posargs(self) -> List[str]:
-        """Any extra arguments from the ``nox`` commandline or :class:`Session.notify`."""
+    def posargs(self) -> list[str]:
+        """Any extra arguments from the ``nox`` commandline or :class:`Session.notify`.
+        """
         return self._runner.posargs
 
     @property
@@ -181,12 +192,12 @@ class Session:
         return venv
 
     @property
-    def python(self) -> Optional[Union[str, Sequence[str], bool]]:
+    def python(self) -> str | Sequence[str] | bool | None:
         """The python version passed into ``@nox.session``."""
         return self._runner.func.python
 
     @property
-    def bin_paths(self) -> Optional[List[str]]:
+    def bin_paths(self) -> list[str] | None:
         """The bin directories for the virtualenv."""
         return self.virtualenv.bin_paths
 
@@ -214,7 +225,8 @@ class Session:
 
     @property
     def interactive(self) -> bool:
-        """Returns True if Nox is being run in an interactive session or False otherwise."""
+        """Returns True if Nox is being run in an interactive session or False otherwise.
+        """
         return not self._runner.global_config.non_interactive and sys.stdin.isatty()
 
     @property
@@ -227,9 +239,9 @@ class Session:
         to the Noxfile's directory before running any sessions. This gives
         you the original working directory that Nox was invoked form.
         """
-        return self._runner.global_config.invoked_from
+        return self._runner.global_config.invoked_from  # type: ignore[no-any-return]
 
-    def chdir(self, dir: Union[str, os.PathLike]) -> _WorkingDirContext:
+    def chdir(self, dir: str | os.PathLike[str]) -> _WorkingDirContext:
         """Change the current working directory.
 
         Can be used as a context manager to automatically restore the working directory::
@@ -249,7 +261,7 @@ class Session:
     """An alias for :meth:`chdir`."""
 
     def _run_func(
-        self, func: Callable, args: Iterable[Any], kwargs: Mapping[str, Any]
+        self, func: Callable[..., Any], args: Iterable[Any], kwargs: Mapping[str, Any]
     ) -> Any:
         """Legacy support for running a function through :func`run`."""
         self.log(f"{func}(args={args!r}, kwargs={kwargs!r})")
@@ -257,11 +269,11 @@ class Session:
             return func(*args, **kwargs)
         except Exception as e:
             logger.exception(f"Function {func!r} raised {e!r}.")
-            raise nox.command.CommandFailed()
+            raise nox.command.CommandFailed() from e
 
     def run(
-        self, *args: str, env: Optional[Mapping[str, str]] = None, **kwargs: Any
-    ) -> Optional[Any]:
+        self, *args: str, env: Mapping[str, str] | None = None, **kwargs: Any
+    ) -> Any | None:
         """Run a command.
 
         Commands must be specified as a list of strings, for example::
@@ -280,7 +292,18 @@ class Session:
                 'bash', '-c', 'echo $SOME_ENV',
                 env={'SOME_ENV': 'Hello'})
 
-        You can also tell nox to treat non-zero exit codes as success using
+        You can extend the shutdown timeout to allow long-running cleanup tasks to
+        complete before being terminated. For example, if you wanted to allow ``pytest``
+        extra time to clean up large projects in the case that Nox receives an
+        interrupt signal from your build system and needs to terminate its child
+        processes::
+
+            session.run(
+                'pytest', '-k', 'long_cleanup',
+                interrupt_timeout=10.0,
+                terminate_timeout=2.0)
+
+        You can also tell Nox to treat non-zero exit codes as success using
         ``success_codes``. For example, if you wanted to treat the ``pytest``
         "tests discovered, but none selected" error as success::
 
@@ -303,10 +326,21 @@ class Session:
                # Display coverage report even when tests fail.
                session.run("coverage", "report")
 
+        If you pass ``silent=True``, you can capture the output of a command that would
+        otherwise be shown to the user. For example to get the current Git commit ID::
+
+            out = session.run(
+                "git", "rev-parse", "--short", "HEAD",
+                external=True, silent=True
+            )
+
+            print("Current Git commit is", out.strip())
+
         :param env: A dictionary of environment variables to expose to the
             command. By default, all environment variables are passed.
         :type env: dict or None
         :param bool silent: Silence command output, unless the command fails.
+            If ``True``, returns the command output (unless the command fails).
             ``False`` by default.
         :param success_codes: A list of return codes that are considered
             successful. By default, only ``0`` is considered success.
@@ -317,6 +351,22 @@ class Session:
             ``--error-on-external-run``. This has no effect for sessions that
             do not have a virtualenv.
         :type external: bool
+        :param interrupt_timeout: The timeout (in seconds) that Nox should wait after it
+            and its children receive an interrupt signal before sending a terminate
+            signal to its children. Set to ``None`` to never send a terminate signal.
+            Default: ``0.3``
+        :type interrupt_timeout: float or None
+        :param terminate_timeout: The timeout (in seconds) that Nox should wait after it
+            sends a terminate signal to its children before sending a kill signal to
+            them. Set to ``None`` to never send a kill signal.
+            Default: ``0.2``
+        :type terminate_timeout: float or None
+        :param stdout: Redirect standard output of the command into a file. Can't be
+            combined with *silent*.
+        :type stdout: file or file descriptor
+        :param stderr: Redirect standard output of the command into a file. Can't be
+            combined with *silent*.
+        :type stderr: file or file descriptor
         """
         if not args:
             raise ValueError("At least one argument required to run().")
@@ -328,8 +378,8 @@ class Session:
         return self._run(*args, env=env, **kwargs)
 
     def run_always(
-        self, *args: str, env: Optional[Mapping[str, str]] = None, **kwargs: Any
-    ) -> Optional[Any]:
+        self, *args: str, env: Mapping[str, str] | None = None, **kwargs: Any
+    ) -> Any | None:
         """Run a command **always**.
 
         This is a variant of :meth:`run` that runs even in the presence of
@@ -357,6 +407,16 @@ class Session:
             ``--error-on-external-run``. This has no effect for sessions that
             do not have a virtualenv.
         :type external: bool
+        :param interrupt_timeout: The timeout (in seconds) that Nox should wait after it
+            and its children receive an interrupt signal before sending a terminate
+            signal to its children. Set to ``None`` to never send a terminate signal.
+            Default: ``0.3``
+        :type interrupt_timeout: float or None
+        :param terminate_timeout: The timeout (in seconds) that Nox should wait after it
+            sends a terminate signal to its children before sending a kill signal to
+            them. Set to ``None`` to never send a kill signal.
+            Default: ``0.2``
+        :type terminate_timeout: float or None
         """
         if (
             self._runner.global_config.no_install
@@ -371,7 +431,7 @@ class Session:
         return self._run(*args, env=env, **kwargs)
 
     def _run(
-        self, *args: str, env: Optional[Mapping[str, str]] = None, **kwargs: Any
+        self, *args: str, env: Mapping[str, str] | None = None, **kwargs: Any
     ) -> Any:
         """Like run(), except that it runs even if --install-only is provided."""
         # Legacy support - run a function given.
@@ -404,7 +464,7 @@ class Session:
         self,
         *args: str,
         auto_offline: bool = True,
-        channel: Union[str, Sequence[str]] = "",
+        channel: str | Sequence[str] = "",
         **kwargs: Any,
     ) -> None:
         """Install invokes `conda install`_ to install packages inside of the
@@ -432,7 +492,7 @@ class Session:
             # Install in editable mode.
             session.install('-e', '.', '--no-deps')
 
-        You can specify a conda channel using `channel=`; a falsy value will
+        You can specify a conda channel using `channel=`; a falsey value will
         not change the current channels. You can specify a list of channels if
         needed.
 
@@ -442,12 +502,13 @@ class Session:
         """
         venv = self._runner.venv
 
-        prefix_args: Tuple[str, ...] = ()
+        prefix_args: tuple[str, ...] = ()
         if isinstance(venv, CondaEnv):
             prefix_args = ("--prefix", venv.location)
         elif not isinstance(venv, PassthroughEnv):  # pragma: no cover
             raise ValueError(
-                "A session without a conda environment can not install dependencies from conda."
+                "A session without a conda environment can not install dependencies"
+                " from conda."
             )
 
         if not args:
@@ -462,10 +523,11 @@ class Session:
         if "silent" not in kwargs:
             kwargs["silent"] = True
 
-        extraopts: List[str] = []
+        extraopts: list[str] = []
         if auto_offline and venv.is_offline():
             logger.warning(
-                "Automatically setting the `--offline` flag as conda repo seems unreachable."
+                "Automatically setting the `--offline` flag as conda repo seems"
+                " unreachable."
             )
             extraopts.append("--offline")
 
@@ -509,6 +571,12 @@ class Session:
 
         Additional keyword args are the same as for :meth:`run`.
 
+        .. warning::
+
+            Running ``session.install`` without a virtual environment
+            is no longer supported. If you still want to do that, please
+            use ``session.run("pip", "install", ...)`` instead.
+
         .. _pip: https://pip.readthedocs.org
         """
         venv = self._runner.venv
@@ -520,13 +588,11 @@ class Session:
                 "A session without a virtualenv can not install dependencies."
             )
         if isinstance(venv, PassthroughEnv):
-            warnings.warn(
-                f"Session {self.name} does not have a virtual environment, "
-                "so use of session.install() is deprecated since it would modify "
-                "the global Python environment. If you're really sure that is "
-                'what you want to do, use session.run("pip", "install", ...) instead.',
-                category=FutureWarning,
-                stacklevel=2,
+            raise ValueError(
+                f"Session {self.name} does not have a virtual environment, so use of"
+                " session.install() is no longer allowed since it would modify the"
+                " global Python environment. If you're really sure that is what you"
+                ' want to do, use session.run("pip", "install", ...) instead.'
             )
         if not args:
             raise ValueError("At least one argument required to install().")
@@ -541,8 +607,8 @@ class Session:
 
     def notify(
         self,
-        target: "Union[str, SessionRunner]",
-        posargs: Optional[Iterable[str]] = None,
+        target: str | SessionRunner,
+        posargs: Iterable[str] | None = None,
     ) -> None:
         """Place the given session at the end of the queue.
 
@@ -588,11 +654,11 @@ class Session:
         """Outputs a debug-level message during the session."""
         logger.debug(*args, **kwargs)
 
-    def error(self, *args: Any) -> "_typing.NoReturn":
+    def error(self, *args: Any) -> NoReturn:
         """Immediately aborts the session and optionally logs an error."""
         raise _SessionQuit(*args)
 
-    def skip(self, *args: Any) -> "_typing.NoReturn":
+    def skip(self, *args: Any) -> NoReturn:
         """Immediately skips the session and optionally logs a warning."""
         raise _SessionSkip(*args)
 
@@ -601,21 +667,21 @@ class SessionRunner:
     def __init__(
         self,
         name: str,
-        signatures: List[str],
+        signatures: list[str],
         func: Func,
         global_config: argparse.Namespace,
-        manifest: "Manifest",
+        manifest: Manifest,
     ) -> None:
         self.name = name
         self.signatures = signatures
         self.func = func
         self.global_config = global_config
         self.manifest = manifest
-        self.venv: Optional[ProcessEnv] = None
-        self.posargs: List[str] = global_config.posargs[:]
+        self.venv: ProcessEnv | None = None
+        self.posargs: list[str] = global_config.posargs[:]
 
     @property
-    def description(self) -> Optional[str]:
+    def description(self) -> str | None:
         doc = self.func.__doc__
         if doc:
             first_line = doc.strip().split("\n")[0]
@@ -630,6 +696,10 @@ class SessionRunner:
     def friendly_name(self) -> str:
         return self.signatures[0] if self.signatures else self.name
 
+    @property
+    def tags(self) -> list[str]:
+        return self.func.tags
+
     @property
     def envdir(self) -> str:
         return _normalize_path(self.global_config.envdir, self.friendly_name)
@@ -674,24 +744,22 @@ class SessionRunner:
             )
         else:
             raise ValueError(
-                f"Expected venv_backend one of ('virtualenv', 'conda', 'mamba', 'venv'), but got '{backend}'."
+                "Expected venv_backend one of ('virtualenv', 'conda', 'mamba',"
+                f" 'venv'), but got '{backend}'."
             )
 
         self.venv.create()
 
-    def execute(self) -> "Result":
+    def execute(self) -> Result:
         logger.warning(f"Running session {self.friendly_name}")
 
         try:
-            # By default, nox should quietly change to the directory where
-            # the noxfile.py file is located.
-            cwd = py.path.local(
-                os.path.realpath(os.path.dirname(self.global_config.noxfile))
-            ).as_cwd()
+            cwd = os.path.realpath(os.path.dirname(self.global_config.noxfile))
 
-            with cwd:
+            with _chdir(cwd):
                 self._create_venv()
                 session = Session(self)
+                session.env["NOX_CURRENT_SESSION"] = session.name
                 self.func(session)
 
             # Nothing went wrong; return a success.
@@ -701,6 +769,9 @@ class SessionRunner:
             if self.global_config.error_on_missing_interpreters:
                 return Result(self, Status.FAILED, reason=str(exc))
             else:
+                logger.warning(
+                    "Missing interpreters will error by default on CI systems."
+                )
                 return Result(self, Status.SKIPPED, reason=str(exc))
 
         except _SessionQuit as exc:
@@ -725,7 +796,7 @@ class Result:
     """An object representing the result of a session."""
 
     def __init__(
-        self, session: SessionRunner, status: Status, reason: Optional[str] = None
+        self, session: SessionRunner, status: Status, reason: str | None = None
     ) -> None:
         """Initialize the Result object.
 
@@ -754,11 +825,12 @@ class Result:
         """
         if self.status == Status.SUCCESS:
             return "was successful"
+
         status = self.status.name.lower()
         if self.reason:
             return f"{status}: {self.reason}"
-        else:
-            return status
+
+        return status
 
     def log(self, message: str) -> None:
         """Log a message using the appropriate log function.
@@ -775,7 +847,7 @@ class Result:
             log_function = logger.error
         log_function(message)
 
-    def serialize(self) -> Dict[str, Any]:
+    def serialize(self) -> dict[str, Any]:
         """Return a serialized representation of this result.
 
         Returns:
