@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 import ast
 import importlib.util
 import json
@@ -19,7 +21,6 @@ import os
 import sys
 import types
 from argparse import Namespace
-from typing import List, Union
 
 from colorlog.escape_codes import parse_colors
 
@@ -33,41 +34,38 @@ from nox.sessions import Result
 
 def _load_and_exec_nox_module(global_config: Namespace) -> types.ModuleType:
     """
-    Loads, executes, then returns the global_config nox module.
+    Loads, executes, then returns the global_config Nox module.
 
     Args:
         global_config (Namespace): The global config.
 
     Raises:
-        IOError: If the nox module cannot be loaded. This
+        IOError: If the Nox module cannot be loaded. This
             exception is chosen such that it will be caught
             by load_nox_module and logged appropriately.
 
     Returns:
-        types.ModuleType: The initialised nox module.
+        types.ModuleType: The initialised Nox module.
     """
     spec = importlib.util.spec_from_file_location(
         "user_nox_module", global_config.noxfile
     )
-    if not spec:
-        raise OSError(f"Could not get module spec from {global_config.noxfile}")
+    assert spec is not None  # If None, fatal importlib error, would crash anyway
 
     module = importlib.util.module_from_spec(spec)
-    if not module:
-        raise OSError(f"Noxfile {global_config.noxfile} is not a valid python module.")
+    assert module is not None  # If None, fatal importlib error, would crash anyway
 
     sys.modules["user_nox_module"] = module
 
     loader = spec.loader
-    if not loader:  # pragma: no cover
-        raise OSError(f"Could not get module loader for {global_config.noxfile}")
+    assert loader is not None  # If None, fatal importlib error, would crash anyway
     # See https://docs.python.org/3/library/importlib.html#importing-a-source-file-directly
-    loader.exec_module(module)  # type: ignore[attr-defined]
+    loader.exec_module(module)
     return module
 
 
-def load_nox_module(global_config: Namespace) -> Union[types.ModuleType, int]:
-    """Load the user's noxfile and return the module object for it.
+def load_nox_module(global_config: Namespace) -> types.ModuleType | int:
+    """Load the user's Noxfile and return the module object for it.
 
     .. note::
 
@@ -82,7 +80,7 @@ def load_nox_module(global_config: Namespace) -> Union[types.ModuleType, int]:
     """
     try:
         # Save the absolute path to the Noxfile.
-        # This will inoculate it if nox changes paths because of an implicit
+        # This will inoculate it if Nox changes paths because of an implicit
         # or explicit chdir (like the one below).
         global_config.noxfile = os.path.realpath(
             # Be sure to expand variables
@@ -100,8 +98,6 @@ def load_nox_module(global_config: Namespace) -> Union[types.ModuleType, int]:
         # invoked from) gets stored by the .invoke_from "option" in _options.
         os.chdir(noxfile_parent_dir)
 
-        return _load_and_exec_nox_module(global_config)
-
     except (VersionCheckFailed, InvalidVersionSpecifier) as error:
         logger.error(str(error))
         return 2
@@ -113,6 +109,8 @@ def load_nox_module(global_config: Namespace) -> Union[types.ModuleType, int]:
     except OSError:
         logger.exception(f"Failed to load Noxfile {global_config.noxfile}")
         return 2
+    else:
+        return _load_and_exec_nox_module(global_config)
 
 
 def merge_noxfile_options(
@@ -130,9 +128,9 @@ def merge_noxfile_options(
 
 
 def discover_manifest(
-    module: Union[types.ModuleType, int], global_config: Namespace
+    module: types.ModuleType | int, global_config: Namespace
 ) -> Manifest:
-    """Discover all session functions in the noxfile module.
+    """Discover all session functions in the Noxfile module.
 
     Args:
         module (module): The Noxfile module.
@@ -146,16 +144,14 @@ def discover_manifest(
     # sorted by decorator call time.
     functions = registry.get()
 
-    # Get the docstring from the noxfile
+    # Get the docstring from the Noxfile
     module_docstring = module.__doc__
 
     # Return the final dictionary of session functions.
     return Manifest(functions, global_config, module_docstring)
 
 
-def filter_manifest(
-    manifest: Manifest, global_config: Namespace
-) -> Union[Manifest, int]:
+def filter_manifest(manifest: Manifest, global_config: Namespace) -> Manifest | int:
     """Filter the manifest according to the provided configuration.
 
     Args:
@@ -167,14 +163,14 @@ def filter_manifest(
             the manifest otherwise (to be sent to the next task).
 
     """
-    # Shouldn't happen unless the noxfile is empty
+    # Shouldn't happen unless the Noxfile is empty
     if not manifest:
         logger.error(f"No sessions found in {global_config.noxfile}.")
         return 3
 
     # Filter by the name of any explicit sessions.
     # This can raise KeyError if a specified session does not exist;
-    # log this if it happens. The sessions does not come from the noxfile
+    # log this if it happens. The sessions does not come from the Noxfile
     # if keywords is not empty.
     if global_config.sessions is not None:
         try:
@@ -195,13 +191,21 @@ def filter_manifest(
             logger.error("Python version selection caused no sessions to be selected.")
             return 3
 
+    # Filter by tags.
+    if global_config.tags is not None:
+        manifest.filter_by_tags(global_config.tags)
+        if not manifest and not global_config.list_sessions:
+            logger.error("Tag selection caused no sessions to be selected.")
+            return 3
+
     # Filter by keywords.
     if global_config.keywords:
         try:
             ast.parse(global_config.keywords, mode="eval")
         except SyntaxError:
             logger.error(
-                "Error while collecting sessions: keywords argument must be a Python expression."
+                "Error while collecting sessions: keywords argument must be a Python"
+                " expression."
             )
             return 3
 
@@ -220,7 +224,7 @@ def filter_manifest(
 def _produce_listing(manifest: Manifest, global_config: Namespace) -> None:
     # If the user just asked for a list of sessions, print that
     # and any docstring specified in noxfile.py and be done. This
-    # can also be called if noxfile sessions is an empty list.
+    # can also be called if Noxfile sessions is an empty list.
 
     if manifest.module_docstring:
         print(manifest.module_docstring.strip(), end="\n\n")
@@ -255,13 +259,12 @@ def _produce_listing(manifest: Manifest, global_config: Namespace) -> None:
         )
 
     print(
-        f"\nsessions marked with {selected_color}*{reset} are selected, sessions marked with {skipped_color}-{reset} are skipped."
+        f"\nsessions marked with {selected_color}*{reset} are selected, sessions marked"
+        f" with {skipped_color}-{reset} are skipped."
     )
 
 
-def honor_list_request(
-    manifest: Manifest, global_config: Namespace
-) -> Union[Manifest, int]:
+def honor_list_request(manifest: Manifest, global_config: Namespace) -> Manifest | int:
     """If --list was passed, simply list the manifest and exit cleanly.
 
     Args:
@@ -280,7 +283,7 @@ def honor_list_request(
     return 0
 
 
-def run_manifest(manifest: Manifest, global_config: Namespace) -> List[Result]:
+def run_manifest(manifest: Manifest, global_config: Namespace) -> list[Result]:
     """Run the full manifest of sessions.
 
     Args:
@@ -302,7 +305,8 @@ def run_manifest(manifest: Manifest, global_config: Namespace) -> List[Result]:
         if WARN_PYTHONS_IGNORED in session.func.should_warn:
             logger.warning(
                 f"Session {session.name} is set to run with venv_backend='none', "
-                f"IGNORING its python={session.func.should_warn[WARN_PYTHONS_IGNORED]} parametrization. "
+                "IGNORING its"
+                f" python={session.func.should_warn[WARN_PYTHONS_IGNORED]} parametrization. "
             )
 
         result = session.execute()
@@ -320,7 +324,7 @@ def run_manifest(manifest: Manifest, global_config: Namespace) -> List[Result]:
     return results
 
 
-def print_summary(results: List[Result], global_config: Namespace) -> List[Result]:
+def print_summary(results: list[Result], global_config: Namespace) -> list[Result]:
     """Print a summary of the results.
 
     Args:
@@ -347,7 +351,7 @@ def print_summary(results: List[Result], global_config: Namespace) -> List[Resul
     return results
 
 
-def create_report(results: List[Result], global_config: Namespace) -> List[Result]:
+def create_report(results: list[Result], global_config: Namespace) -> list[Result]:
     """Write a report to the location designated in the config, if any.
 
     Args:
@@ -377,7 +381,7 @@ def create_report(results: List[Result], global_config: Namespace) -> List[Resul
     return results
 
 
-def final_reduce(results: List[Result], global_config: Namespace) -> int:
+def final_reduce(results: list[Result], global_config: Namespace) -> int:
     """Reduce the results to a final exit code.
 
     Args:
