@@ -12,22 +12,22 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
+import contextlib
 import os
 import platform
 import re
 import shutil
+import subprocess
 import sys
 from socket import gethostbyname
-from typing import Any, List, Mapping, Optional, Tuple, Union
-
-import py
+from typing import Any, ClassVar, Mapping
 
 import nox
 import nox.command
 from nox.logger import logger
 
-from . import _typing
-
 # Problematic environment variables that are stripped from all commands inside
 # of a virtualenv. See https://github.com/theacodes/nox/issues/44
 _BLACKLISTED_ENV_VARS = frozenset(
@@ -52,10 +52,10 @@ class ProcessEnv:
     is_sandboxed = False
 
     # Special programs that aren't included in the environment.
-    allowed_globals: _typing.ClassVar[Tuple[Any, ...]] = ()
+    allowed_globals: ClassVar[tuple[Any, ...]] = ()
 
     def __init__(
-        self, bin_paths: None = None, env: Optional[Mapping[str, str]] = None
+        self, bin_paths: None = None, env: Mapping[str, str] | None = None
     ) -> None:
         self._bin_paths = bin_paths
         self.env = os.environ.copy()
@@ -73,7 +73,7 @@ class ProcessEnv:
             )
 
     @property
-    def bin_paths(self) -> Optional[List[str]]:
+    def bin_paths(self) -> list[str] | None:
         return self._bin_paths
 
     @property
@@ -88,7 +88,7 @@ class ProcessEnv:
         raise NotImplementedError("ProcessEnv.create should be overwritten in subclass")
 
 
-def locate_via_py(version: str) -> Optional[str]:
+def locate_via_py(version: str) -> str | None:
     """Find the Python executable using the Windows Launcher.
 
     This is based on :pep:397 which details that executing
@@ -107,16 +107,20 @@ def locate_via_py(version: str) -> Optional[str]:
         if it is found.
     """
     script = "import sys; print(sys.executable)"
-    py_exe = py.path.local.sysfind("py")
+    py_exe = shutil.which("py")
     if py_exe is not None:
-        try:
-            return py_exe.sysexec("-" + version, "-c", script).strip()
-        except py.process.cmdexec.Error:
-            return None
+        ret = subprocess.run(
+            [py_exe, f"-{version}", "-c", script],
+            check=False,
+            text=True,
+            capture_output=True,
+        )
+        if ret.returncode == 0 and ret.stdout:
+            return ret.stdout.strip()
     return None
 
 
-def locate_using_path_and_version(version: str) -> Optional[str]:
+def locate_using_path_and_version(version: str) -> str | None:
     """Check the PATH's python interpreter and return it if the version
     matches.
 
@@ -136,21 +140,23 @@ def locate_using_path_and_version(version: str) -> Optional[str]:
         return None
 
     script = "import platform; print(platform.python_version())"
-    path_python = py.path.local.sysfind("python")
+    path_python = shutil.which("python")
     if path_python:
-        try:
-            prefix = f"{version}."
-            version_string = path_python.sysexec("-c", script).strip()
-            if version_string.startswith(prefix):
-                return str(path_python)
-        except py.process.cmdexec.Error:
-            return None
+        prefix = f"{version}"
+        ret = subprocess.run(
+            [path_python, "-c", script],
+            check=False,
+            text=True,
+            capture_output=True,
+        )
+        if ret.returncode == 0 and ret.stdout and ret.stdout.strip().startswith(prefix):
+            return path_python
 
     return None
 
 
 class PassthroughEnv(ProcessEnv):
-    """Represents the environment used to run nox itself
+    """Represents the environment used to run Nox itself
 
     For now, this class is empty but it might contain tools to grasp some
     hints about the actual env.
@@ -191,7 +197,7 @@ class CondaEnv(ProcessEnv):
     def __init__(
         self,
         location: str,
-        interpreter: Optional[str] = None,
+        interpreter: str | None = None,
         reuse_existing: bool = False,
         venv_params: Any = None,
         *,
@@ -201,7 +207,7 @@ class CondaEnv(ProcessEnv):
         self.location = os.path.abspath(location)
         self.interpreter = interpreter
         self.reuse_existing = reuse_existing
-        self.venv_params = venv_params if venv_params else []
+        self.venv_params = venv_params or []
         self.conda_cmd = conda_cmd
         super().__init__(env={"CONDA_PREFIX": self.location})
 
@@ -221,15 +227,13 @@ class CondaEnv(ProcessEnv):
                 ]
                 nox.command.run(cmd, silent=True, log=False)
                 # Make sure that location is clean
-                try:
+                with contextlib.suppress(FileNotFoundError):
                     shutil.rmtree(self.location)
-                except FileNotFoundError:
-                    pass
 
         return True
 
     @property
-    def bin_paths(self) -> List[str]:
+    def bin_paths(self) -> list[str]:
         """Returns the location of the conda env's bin folder."""
         # see https://github.com/conda/conda/blob/f60f0f1643af04ed9a51da3dd4fa242de81e32f4/conda/activate.py#L563-L572
         if _SYSTEM == "Windows":
@@ -241,8 +245,8 @@ class CondaEnv(ProcessEnv):
                 os.path.join(self.location, "Scripts"),
                 os.path.join(self.location, "bin"),
             ]
-        else:
-            return [os.path.join(self.location, "bin")]
+
+        return [os.path.join(self.location, "bin")]
 
     def create(self) -> bool:
         """Create the conda env."""
@@ -314,7 +318,7 @@ class VirtualEnv(ProcessEnv):
     def __init__(
         self,
         location: str,
-        interpreter: Optional[str] = None,
+        interpreter: str | None = None,
         reuse_existing: bool = False,
         *,
         venv: bool = False,
@@ -323,10 +327,10 @@ class VirtualEnv(ProcessEnv):
         self.location_name = location
         self.location = os.path.abspath(location)
         self.interpreter = interpreter
-        self._resolved: Union[None, str, InterpreterNotFound] = None
+        self._resolved: None | str | InterpreterNotFound = None
         self.reuse_existing = reuse_existing
         self.venv_or_virtualenv = "venv" if venv else "virtualenv"
-        self.venv_params = venv_params if venv_params else []
+        self.venv_params = venv_params or []
         super().__init__(env={"VIRTUAL_ENV": self.location})
 
     def _clean_location(self) -> bool:
@@ -379,7 +383,7 @@ class VirtualEnv(ProcessEnv):
 
         return original == created
 
-    def _read_base_prefix_from_pyvenv_cfg(self) -> Optional[str]:
+    def _read_base_prefix_from_pyvenv_cfg(self) -> str | None:
         """Return the base-prefix entry from pyvenv.cfg, if present."""
         path = os.path.join(self.location, "pyvenv.cfg")
         if os.path.isfile(path):
@@ -422,7 +426,7 @@ class VirtualEnv(ProcessEnv):
             cleaned_interpreter = f"python{xy_version}"
 
         # If the cleaned interpreter is on the PATH, go ahead and return it.
-        if py.path.local.sysfind(cleaned_interpreter):
+        if shutil.which(cleaned_interpreter):
             self._resolved = cleaned_interpreter
             return self._resolved
 
@@ -455,12 +459,11 @@ class VirtualEnv(ProcessEnv):
         raise self._resolved
 
     @property
-    def bin_paths(self) -> List[str]:
+    def bin_paths(self) -> list[str]:
         """Returns the location of the virtualenv's bin folder."""
         if _SYSTEM == "Windows":
             return [os.path.join(self.location, "Scripts")]
-        else:
-            return [os.path.join(self.location, "bin")]
+        return [os.path.join(self.location, "bin")]
 
     def create(self) -> bool:
         """Create the virtualenv or venv."""
@@ -484,7 +487,8 @@ class VirtualEnv(ProcessEnv):
         resolved_interpreter_name = os.path.basename(self._resolved_interpreter)
 
         logger.info(
-            f"Creating virtual environment ({self.venv_or_virtualenv}) using {resolved_interpreter_name} in {self.location_name}"
+            f"Creating virtual environment ({self.venv_or_virtualenv}) using"
+            f" {resolved_interpreter_name} in {self.location_name}"
         )
         nox.command.run(cmd, silent=True, log=nox.options.verbose or False)
 
