@@ -13,6 +13,8 @@
 # limitations under the License.
 
 
+from __future__ import annotations
+
 import functools
 import os
 import platform
@@ -29,86 +31,63 @@ if shutil.which("conda"):
     nox.options.sessions.append("conda_tests")
 
 
-def is_python_version(session, version):
-    if not version.startswith(session.python):
-        return False
-    py_version = session.run("python", "-V", silent=True)
-    py_version = py_version.partition(" ")[2].strip()
-    return py_version.startswith(version)
-
-
-@nox.session(python=["3.6", "3.7", "3.8", "3.9", "3.10"])
-def tests(session):
+@nox.session(python=["3.7", "3.8", "3.9", "3.10", "3.11"])
+def tests(session: nox.Session) -> None:
     """Run test suite with pytest."""
-    session.create_tmp()
+    session.create_tmp()  # Fixes permission errors on Windows
     session.install("-r", "requirements-test.txt")
     session.install("-e", ".[tox_to_nox]")
-    tests = session.posargs or ["tests/"]
-    if is_python_version(session, "3.6.0"):
-        session.run("pytest", *tests)
-        return
     session.run(
         "pytest",
         "--cov=nox",
         "--cov-config",
         "pyproject.toml",
         "--cov-report=",
-        *tests,
+        *session.posargs,
         env={"COVERAGE_FILE": f".coverage.{session.python}"},
     )
     session.notify("cover")
 
 
-@nox.session(python=["3.6", "3.7", "3.8", "3.9", "3.10"], venv_backend="conda")
-def conda_tests(session):
+@nox.session(python=["3.7", "3.8", "3.9", "3.10"], venv_backend="conda")
+def conda_tests(session: nox.Session) -> None:
     """Run test suite with pytest."""
-    session.create_tmp()
+    session.create_tmp()  # Fixes permission errors on Windows
     session.conda_install(
         "--file", "requirements-conda-test.txt", "--channel", "conda-forge"
     )
     session.install("-e", ".", "--no-deps")
-    tests = session.posargs or ["tests/"]
-    session.run("pytest", *tests)
+    session.run("pytest", *session.posargs)
 
 
 @nox.session
-def cover(session):
+def cover(session: nox.Session) -> None:
     """Coverage analysis."""
     if ON_WINDOWS_CI:
         return
 
-    # 3.10 produces different coverage results for some reason
-    # see https://github.com/theacodes/nox/issues/478
-    fail_under = 100
-    py_version = sys.version_info
-    if py_version.major == 3 and py_version.minor == 10:
-        fail_under = 99
-
     session.install("coverage[toml]")
     session.run("coverage", "combine")
-    session.run("coverage", "report", f"--fail-under={fail_under}", "--show-missing")
+    session.run("coverage", "report", "--fail-under=100", "--show-missing")
     session.run("coverage", "erase")
 
 
 @nox.session(python="3.9")
-def lint(session):
+def lint(session: nox.Session) -> None:
     """Run pre-commit linting."""
     session.install("pre-commit")
-    # See https://github.com/theacodes/nox/issues/545
-    # and https://github.com/pre-commit/pre-commit/issues/2178#issuecomment-1002163763
     session.run(
         "pre-commit",
         "run",
         "--all-files",
         "--show-diff-on-failure",
         "--hook-stage=manual",
-        env={"SETUPTOOLS_USE_DISTUTILS": "stdlib"},
         *session.posargs,
     )
 
 
 @nox.session
-def docs(session):
+def docs(session: nox.Session) -> None:
     """Build the documentation."""
     output_dir = os.path.join(session.create_tmp(), "output")
     doctrees, html = map(
@@ -127,3 +106,55 @@ def docs(session):
         sphinx_args.insert(0, "--open-browser")
 
     session.run(sphinx_cmd, *sphinx_args)
+
+
+# The following sessions are only to be run in CI to check the nox GHA action
+def _check_python_version(session: nox.Session) -> None:
+    if session.python.startswith("pypy"):
+        python_version = session.python[4:]
+        implementation = "pypy"
+    else:
+        python_version = session.python
+        implementation = "cpython"
+    session.run(
+        "python",
+        "-c",
+        "import sys; assert '.'.join(str(v) for v in sys.version_info[:2]) =="
+        f" '{python_version}'",
+    )
+    if python_version[:2] != "2.":
+        session.run(
+            "python",
+            "-c",
+            f"import sys; assert sys.implementation.name == '{implementation}'",
+        )
+
+
+@nox.session(python=["3.7", "3.8", "3.9", "3.10", "pypy3.7", "pypy3.8", "pypy3.9"])
+def github_actions_default_tests(session: nox.Session) -> None:
+    """Check default versions installed by the nox GHA Action"""
+    assert sys.version_info[:2] == (3, 10)
+    _check_python_version(session)
+
+
+# The following sessions are only to be run in CI to check the nox GHA action
+@nox.session(
+    python=[
+        "2.7",
+        "3.4",
+        "3.5",
+        "3.6",
+        "3.7",
+        "3.8",
+        "3.9",
+        "3.10",
+        "3.11",
+        "pypy2.7",
+        "pypy3.7",
+        "pypy3.8",
+        "pypy3.9",
+    ]
+)
+def github_actions_all_tests(session: nox.Session) -> None:
+    """Check all versions installed by the nox GHA Action"""
+    _check_python_version(session)
