@@ -12,14 +12,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 import ctypes
 import logging
 import os
 import platform
+import shutil
 import signal
 import subprocess
 import sys
 import time
+from pathlib import Path
 from textwrap import dedent
 from unittest import mock
 
@@ -55,6 +59,13 @@ def test_run_silent(capsys):
     assert out == ""
 
 
+@pytest.mark.skipif(shutil.which("git") is None, reason="Needs git")
+def test_run_not_in_path(capsys):
+    # Paths falls back on the environment PATH if the command is not found.
+    result = nox.command.run(["git", "--version"], paths=["."])
+    assert result is True
+
+
 def test_run_verbosity(capsys, caplog):
     caplog.clear()
     with caplog.at_level(logging.DEBUG):
@@ -110,6 +121,19 @@ def test_run_verbosity_failed_command(capsys, caplog):
     assert not logs
 
 
+@pytest.mark.skipif(
+    platform.system() == "Windows",
+    reason="See https://github.com/python/cpython/issues/85815",
+)
+def test_run_non_str():
+    result = nox.command.run(
+        [Path(PYTHON), "-c", "import sys; print(sys.argv)", Path(PYTHON)],
+        silent=True,
+    )
+
+    assert PYTHON in result
+
+
 def test_run_env_unicode():
     result = nox.command.run(
         [PYTHON, "-c", 'import os; print(os.environ["SIGIL"])'],
@@ -145,15 +169,19 @@ def test_run_path_nonexistent():
     assert "/non/existent" not in result
 
 
-def test_run_path_existent(tmpdir, monkeypatch):
-    executable = tmpdir.join("testexc")
-    executable.ensure("")
+def test_run_path_existent(tmp_path: Path):
+    executable_name = (
+        "testexc.exe" if "windows" in platform.platform().lower() else "testexc"
+    )
+    tmp_path.touch()
+    executable = tmp_path.joinpath(executable_name)
+    executable.touch()
     executable.chmod(0o700)
 
     with mock.patch("nox.command.popen") as mock_command:
         mock_command.return_value = (0, "")
-        nox.command.run(["testexc"], silent=True, paths=[tmpdir.strpath])
-        mock_command.assert_called_with([executable.strpath], env=None, silent=True)
+        nox.command.run([executable_name], silent=True, paths=[str(tmp_path)])
+        mock_command.assert_called_with([str(executable)], env=None, silent=True)
 
 
 def test_run_external_warns(tmpdir, caplog):
@@ -202,8 +230,10 @@ def test_fail_with_silent(capsys):
             [
                 PYTHON,
                 "-c",
-                'import sys; sys.stdout.write("out");'
-                'sys.stderr.write("err"); sys.exit(1)',
+                (
+                    'import sys; sys.stdout.write("out");'
+                    'sys.stderr.write("err"); sys.exit(1)'
+                ),
             ],
             silent=True,
         )
@@ -287,18 +317,13 @@ def format_program(program, marker):
 def run_pytest_in_new_console_session(test):
     """Run the given test in a separate console session."""
     env = dict(os.environ, SECONDARY_CONSOLE_SESSION="")
-    creationflags = (
-        subprocess.CREATE_NO_WINDOW
-        if sys.version_info[:2] >= (3, 7)
-        else subprocess.CREATE_NEW_CONSOLE
-    )
+    creationflags = subprocess.CREATE_NO_WINDOW
 
     subprocess.run(
         [sys.executable, "-m", "pytest", f"{__file__}::{test}"],
         env=env,
         check=True,
-        stdout=subprocess.PIPE,
-        stderr=subprocess.PIPE,
+        capture_output=True,
         creationflags=creationflags,
     )
 
@@ -367,8 +392,10 @@ def test_custom_stdout(capsys, tmpdir):
             [
                 PYTHON,
                 "-c",
-                'import sys; sys.stdout.write("out");'
-                'sys.stderr.write("err"); sys.exit(0)',
+                (
+                    'import sys; sys.stdout.write("out");'
+                    'sys.stderr.write("err"); sys.exit(0)'
+                ),
             ],
             stdout=stdout,
         )
@@ -395,8 +422,10 @@ def test_custom_stdout_failed_command(capsys, tmpdir):
                 [
                     PYTHON,
                     "-c",
-                    'import sys; sys.stdout.write("out");'
-                    'sys.stderr.write("err"); sys.exit(1)',
+                    (
+                        'import sys; sys.stdout.write("out");'
+                        'sys.stderr.write("err"); sys.exit(1)'
+                    ),
                 ],
                 stdout=stdout,
             )
@@ -416,8 +445,10 @@ def test_custom_stderr(capsys, tmpdir):
             [
                 PYTHON,
                 "-c",
-                'import sys; sys.stdout.write("out");'
-                'sys.stderr.write("err"); sys.exit(0)',
+                (
+                    'import sys; sys.stdout.write("out");'
+                    'sys.stderr.write("err"); sys.exit(0)'
+                ),
             ],
             stderr=stderr,
         )
@@ -438,8 +469,10 @@ def test_custom_stderr_failed_command(capsys, tmpdir):
                 [
                     PYTHON,
                     "-c",
-                    'import sys; sys.stdout.write("out");'
-                    'sys.stderr.write("err"); sys.exit(1)',
+                    (
+                        'import sys; sys.stdout.write("out");'
+                        'sys.stderr.write("err"); sys.exit(1)'
+                    ),
                 ],
                 stderr=stderr,
             )
