@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 import contextlib
 import os
 import sys
@@ -37,7 +39,7 @@ VERSION = metadata.version("nox")
 
 
 # This is needed because CI systems will mess up these tests due to the
-# way nox handles the --session parameter's default value. This avoids that
+# way Nox handles the --session parameter's default value. This avoids that
 # mess.
 os.environ.pop("NOXSESSION", None)
 
@@ -656,12 +658,12 @@ def test_main_color_conflict(capsys, monkeypatch):
 
 
 def test_main_force_python(monkeypatch):
-    monkeypatch.setattr(sys, "argv", ["nox", "--force-python=3.10"])
+    monkeypatch.setattr(sys, "argv", ["nox", "--force-python=3.11"])
     with mock.patch("nox.workflow.execute", return_value=0) as execute:
         with mock.patch.object(sys, "exit"):
             nox.__main__.main()
         config = execute.call_args[1]["global_config"]
-    assert config.pythons == config.extra_pythons == ["3.10"]
+    assert config.pythons == config.extra_pythons == ["3.11"]
 
 
 def test_main_reuse_existing_virtualenvs_no_install(monkeypatch):
