@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 import argparse
 import logging
 import operator
@@ -281,7 +283,10 @@ class TestSession:
             session.run(sys.executable, "--version")
 
         run.assert_called_once_with(
-            (sys.executable, "--version"), external=True, env=mock.ANY, paths=None
+            (sys.executable, "--version"),
+            external=True,
+            env=mock.ANY,
+            paths=None,
         )
 
     def test_run_external_condaenv(self):
@@ -342,6 +347,56 @@ class TestSession:
 
         assert session.run_always(operator.add, 23, 19) == 42
 
+    @pytest.mark.parametrize(
+        (
+            "interrupt_timeout_setting",
+            "terminate_timeout_setting",
+            "interrupt_timeout_expected",
+            "terminate_timeout_expected",
+        ),
+        [
+            ("default", "default", 0.3, 0.2),
+            (None, None, None, None),
+            (0, 0, 0, 0),
+            (1, 2, 1, 2),
+        ],
+    )
+    def test_run_shutdown_process_timeouts(
+        self,
+        interrupt_timeout_setting,
+        terminate_timeout_setting,
+        interrupt_timeout_expected,
+        terminate_timeout_expected,
+    ):
+        session, runner = self.make_session_and_runner()
+
+        runner.venv = nox.virtualenv.ProcessEnv()
+
+        subp_popen_instance = mock.Mock()
+        subp_popen_instance.communicate.side_effect = KeyboardInterrupt()
+        with mock.patch(
+            "nox.popen.shutdown_process", autospec=True
+        ) as shutdown_process, mock.patch(
+            "nox.popen.subprocess.Popen",
+            new=mock.Mock(return_value=subp_popen_instance),
+        ):
+            shutdown_process.return_value = ("", "")
+
+            timeout_kwargs = {}
+            if interrupt_timeout_setting != "default":
+                timeout_kwargs["interrupt_timeout"] = interrupt_timeout_setting
+            if terminate_timeout_setting != "default":
+                timeout_kwargs["terminate_timeout"] = terminate_timeout_setting
+
+            with pytest.raises(KeyboardInterrupt):
+                session.run(sys.executable, "--version", **timeout_kwargs)
+
+        shutdown_process.assert_called_once_with(
+            proc=mock.ANY,
+            interrupt_timeout=interrupt_timeout_expected,
+            terminate_timeout=terminate_timeout_expected,
+        )
+
     @pytest.mark.parametrize(
         ("no_install", "reused", "run_called"),
         [
@@ -591,7 +646,7 @@ class TestSession:
                 external="error",
             )
 
-    def test_install_no_venv_deprecated(self):
+    def test_install_no_venv_failure(self):
         runner = nox.sessions.SessionRunner(
             name="test",
             signatures=["test"],
@@ -607,22 +662,14 @@ class TestSession:
 
         session = SessionNoSlots(runner=runner)
 
-        with mock.patch.object(session, "_run", autospec=True) as run:
-            with pytest.warns(
-                FutureWarning,
-                match=r"use of session\.install\(\) is deprecated since it would modify the global Python environment",
-            ):
-                session.install("requests", "urllib3")
-            run.assert_called_once_with(
-                "python",
-                "-m",
-                "pip",
-                "install",
-                "requests",
-                "urllib3",
-                silent=True,
-                external="error",
-            )
+        with pytest.raises(
+            ValueError,
+            match=(
+                r"use of session\.install\(\) is no longer allowed since"
+                r" it would modify the global Python environment"
+            ),
+        ):
+            session.install("requests", "urllib3")
 
     def test_notify(self):
         session, runner = self.make_session_and_runner()
@@ -744,7 +791,7 @@ class TestSessionRunner:
                 envdir="envdir",
                 posargs=[],
                 reuse_existing_virtualenvs=False,
-                error_on_missing_interpreters=False,
+                error_on_missing_interpreters="CI" in os.environ,
             ),
             manifest=mock.create_autospec(nox.manifest.Manifest),
         )
@@ -853,6 +900,8 @@ class TestSessionRunner:
     def make_runner_with_mock_venv(self):
         runner = self.make_runner()
         runner._create_venv = mock.Mock()
+        runner.venv = mock.create_autospec(nox.virtualenv.VirtualEnv)
+        runner.venv.env = {}
         return runner
 
     def test_execute_noop_success(self, caplog):
@@ -899,7 +948,11 @@ class TestSessionRunner:
         assert result.status == nox.sessions.Status.SKIPPED
         assert "no parameters" in result.reason
 
-    def test_execute_skip_missing_interpreter(self):
+    def test_execute_skip_missing_interpreter(self, caplog, monkeypatch):
+        # Important to have this first here as the runner will pick it up
+        # to set default for --error-on-missing-interpreters
+        monkeypatch.delenv("CI", raising=False)
+
         runner = self.make_runner_with_mock_venv()
         runner._create_venv.side_effect = nox.virtualenv.InterpreterNotFound("meep")
 
@@ -907,6 +960,19 @@ class TestSessionRunner:
 
         assert result.status == nox.sessions.Status.SKIPPED
         assert "meep" in result.reason
+        assert (
+            "Missing interpreters will error by default on CI systems." in caplog.text
+        )
+
+    def test_execute_missing_interpreter_on_CI(self, monkeypatch):
+        monkeypatch.setenv("CI", "True")
+        runner = self.make_runner_with_mock_venv()
+        runner._create_venv.side_effect = nox.virtualenv.InterpreterNotFound("meep")
+
+        result = runner.execute()
+
+        assert result.status == nox.sessions.Status.FAILED
+        assert "meep" in result.reason
 
     def test_execute_error_missing_interpreter(self):
         runner = self.make_runner_with_mock_venv()
@@ -953,6 +1019,23 @@ class TestSessionRunner:
 
         assert result.status == nox.sessions.Status.FAILED
 
+    def test_execute_check_env(self):
+        runner = self.make_runner_with_mock_venv()
+
+        def func(session):
+            session.run(
+                sys.executable,
+                "-c",
+                'import os; raise SystemExit(0 if os.environ["NOX_CURRENT_SESSION"] =='
+                f" {session.name!r} else 0)",
+            )
+
+        runner.func = func
+
+        result = runner.execute()
+
+        assert result
+
 
 class TestResult:
     def test_init(self):
