@@ -12,13 +12,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import annotations
+
 import argparse
 import builtins
 import copy
 import json
 import os
 import platform
-from pathlib import Path
 from textwrap import dedent
 from unittest import mock
 
@@ -38,6 +39,7 @@ def session_func():
 session_func.python = None
 session_func.venv_backend = None
 session_func.should_warn = dict()
+session_func.tags = []
 
 
 def session_func_with_python():
@@ -64,7 +66,7 @@ def test_load_nox_module():
 
 
 def test_load_nox_module_expandvars():
-    # Assert that variables are expanded when looking up the path to the noxfile
+    # Assert that variables are expanded when looking up the path to the Noxfile
     # This is particular importand in Windows when one needs to use variables like
     # %TEMP% to point to the noxfile.py
     with mock.patch.dict(os.environ, {"RESOURCES_PATH": RESOURCES}):
@@ -87,56 +89,13 @@ def test_load_nox_module_not_found(caplog, tmp_path):
     )
 
 
-def test_load_nox_module_IOError(caplog):
-
-    # Need to give it a noxfile that exists so load_nox_module can progress
-    # past FileNotFoundError
-    # use our own noxfile.py for this
-    our_noxfile = Path(__file__).parent.parent.joinpath("noxfile.py")
-    config = _options.options.namespace(noxfile=str(our_noxfile))
-
-    with mock.patch("nox.tasks.importlib.util.module_from_spec") as mock_load:
-        mock_load.side_effect = IOError
-
+def test_load_nox_module_os_error(caplog):
+    noxfile = os.path.join(RESOURCES, "noxfile.py")
+    config = _options.options.namespace(noxfile=noxfile)
+    with mock.patch("nox.tasks.check_nox_version", autospec=True) as version_checker:
+        version_checker.side_effect = OSError
         assert tasks.load_nox_module(config) == 2
-        assert "Failed to load Noxfile" in caplog.text
-
-
-def test_load_nox_module_OSError(caplog):
-
-    # Need to give it a noxfile that exists so load_nox_module can progress
-    # past FileNotFoundError
-    # use our own noxfile.py for this
-    our_noxfile = Path(__file__).parent.parent.joinpath("noxfile.py")
-    config = _options.options.namespace(noxfile=str(our_noxfile))
-
-    with mock.patch("nox.tasks.importlib.util.module_from_spec") as mock_load:
-        mock_load.side_effect = OSError
-
-        assert tasks.load_nox_module(config) == 2
-        assert "Failed to load Noxfile" in caplog.text
-
-
-def test_load_nox_module_invalid_spec():
-    our_noxfile = Path(__file__).parent.parent.joinpath("noxfile.py")
-    config = _options.options.namespace(noxfile=str(our_noxfile))
-
-    with mock.patch("nox.tasks.importlib.util.spec_from_file_location") as mock_spec:
-        mock_spec.return_value = None
-
-        with pytest.raises(IOError):
-            tasks._load_and_exec_nox_module(config)
-
-
-def test_load_nox_module_invalid_module():
-    our_noxfile = Path(__file__).parent.parent.joinpath("noxfile.py")
-    config = _options.options.namespace(noxfile=str(our_noxfile))
-
-    with mock.patch("nox.tasks.importlib.util.module_from_spec") as mock_spec:
-        mock_spec.return_value = None
-
-        with pytest.raises(IOError):
-            tasks._load_and_exec_nox_module(config)
+        assert f"Failed to load Noxfile {noxfile}" in caplog.text
 
 
 @pytest.fixture
@@ -285,6 +244,77 @@ def test_filter_manifest_keywords_syntax_error():
     assert return_value == 3
 
 
+@pytest.mark.parametrize(
+    "tags,session_count",
+    [
+        (None, 4),
+        (["foo"], 3),
+        (["bar"], 3),
+        (["baz"], 1),
+        (["foo", "bar"], 4),
+        (["foo", "baz"], 3),
+        (["foo", "bar", "baz"], 4),
+    ],
+)
+def test_filter_manifest_tags(tags, session_count):
+    @nox.session(tags=["foo"])
+    def qux():
+        pass
+
+    @nox.session(tags=["bar"])
+    def quux():
+        pass
+
+    @nox.session(tags=["foo", "bar"])
+    def quuz():
+        pass
+
+    @nox.session(tags=["foo", "bar", "baz"])
+    def corge():
+        pass
+
+    config = _options.options.namespace(
+        sessions=None, pythons=(), posargs=[], tags=tags
+    )
+    manifest = Manifest(
+        {
+            "qux": qux,
+            "quux": quux,
+            "quuz": quuz,
+            "corge": corge,
+        },
+        config,
+    )
+    return_value = tasks.filter_manifest(manifest, config)
+    assert return_value is manifest
+    assert len(manifest) == session_count
+
+
+@pytest.mark.parametrize(
+    "tags",
+    [
+        ["Foo"],
+        ["not-found"],
+    ],
+    ids=[
+        "tags-are-case-insensitive",
+        "tag-does-not-exist",
+    ],
+)
+def test_filter_manifest_tags_not_found(tags, caplog):
+    @nox.session(tags=["foo"])
+    def quux():
+        pass
+
+    config = _options.options.namespace(
+        sessions=None, pythons=(), posargs=[], tags=tags
+    )
+    manifest = Manifest({"quux": quux}, config)
+    return_value = tasks.filter_manifest(manifest, config)
+    assert return_value == 3
+    assert "Tag selection caused no sessions to be selected." in caplog.text
+
+
 def test_honor_list_request_noop():
     config = _options.options.namespace(list_sessions=False)
     manifest = {"thing": mock.sentinel.THING}
